from __future__ import annotations

from amulet_nbt import TAG_Compound

from .anvil_world import AnvilFormat
from amulet.utils.format_utils import check_all_exist, load_leveldat


class AnvilForgeFormat(AnvilFormat):
    """
    This FormatWrapper class is a modification on the :class:`AnvilFormat` class that separates Forge worlds from vanilla worlds.

    Currently there is no extra logic here but this should extend the :class:`AnvilFormat` class to support Forge worlds.
    """

    @staticmethod
    def is_valid(path: str) -> bool:
        if not check_all_exist(path, "level.dat"):
            return False

        try:
            level_dat_root = load_leveldat(path)
            assert isinstance(level_dat_root.value, TAG_Compound)
        except:
            return False

        if "Data" not in level_dat_root:
            return False

        if "FML" in level_dat_root:
            return True

        return False

    @property
    def game_version_string(self) -> str:
        try:
            return f'Java Forge {self.root_tag["Data"]["Version"]["Name"].value}'
        except Exception:
            return f"Java Forge Unknown Version"


export = AnvilForgeFormat



import unittest
import numpy
import copy

from amulet.api.chunk.biomes import Biomes


class BiomeTestCase(unittest.TestCase):
    def test_biomes(self):
        biomes = Biomes()
        biomes.convert_to_3d()
        self.assertEqual(biomes[:, :, :].shape, (4, 64, 4))

        arange = numpy.arange(16).reshape((4, 4))
        biomes[:, 0, :] = arange.reshape((4, 1, 4))

        biomes.convert_to_2d()
        self.assertTrue(numpy.array_equal(biomes[::4, ::4], arange))

    def test_copy(self):
        biomes = Biomes()
        self.assertIsInstance(biomes, Biomes)

        self.assertIsInstance(copy.deepcopy(biomes), Biomes)

        biomes.convert_to_2d()
        self.assertIsInstance(biomes.copy(), Biomes)
        self.assertIsInstance(copy.deepcopy(biomes), Biomes)

        biomes.convert_to_3d()
        self.assertIsInstance(biomes.copy(), Biomes)
        self.assertIsInstance(copy.deepcopy(biomes), Biomes)



import glob
import os
import json

BedrockLevels = []
JavaVanillaLevels = []
JavaForgeLevels = []


java_vanilla_1_12_2 = "java/vanilla/1_12_2"
java_vanilla_1_13 = "java/vanilla/1_13"


def __find_levels():
    this_dir = os.path.dirname(__file__)
    for path in glob.glob(
        os.path.join(this_dir, "**", "world_test_data.json"), recursive=True
    ):
        rel_path = os.path.dirname(os.path.relpath(path, this_dir))
        with open(path) as f:
            test_data = json.load(f)
        platform = test_data["world_data"]["platform"]
        if platform == "java":
            origin = test_data["world_data"]["origin"]
            if origin == "vanilla":
                JavaVanillaLevels.append(rel_path)
            elif origin == "forge":
                JavaForgeLevels.append(rel_path)
            else:
                raise Exception(f"Unknown origin {origin}")
        elif platform == "bedrock":
            BedrockLevels.append(rel_path)
        else:
            raise Exception(f"Unknown platform {platform}")


__find_levels()

JavaLevels = [*JavaVanillaLevels, *JavaForgeLevels]

levels = [
    *BedrockLevels,
    *JavaLevels,
]



import unittest
import os

from amulet.api.block import Block
from amulet.api.chunk import Chunk
from amulet.api.errors import ChunkDoesNotExist
from amulet.api.selection import SelectionBox, SelectionGroup
from amulet import load_level, load_format
from tests.data.util import get_world_path, create_temp_world, clean_temp_world
from amulet.operations.clone import clone
from amulet.operations.delete_chunk import delete_chunk
from amulet.operations.fill import fill
from amulet.operations.replace import replace
from amulet.utils.generator import generator_unpacker
from tests.data import worlds_src
from amulet.level.formats.anvil_world.format import OVERWORLD


class WorldTestBaseCases:
    # Wrapped in another class, so it isn't executed on it's own.

    class WorldTestCase(unittest.TestCase):
        def _setUp(self, world_name):
            self._world_name = world_name
            self.world = load_level(create_temp_world(world_name))

        def tearDown(self):
            self.world.close()
            clean_temp_world(self._world_name)

        def test_get_block(self):
            self.assertEqual(
                "universal_minecraft:air",
                self.world.get_block(0, 0, 0, OVERWORLD).blockstate,
            )
            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )
            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )
            self.assertEqual(
                "universal_minecraft:granite[polished=true]",
                self.world.get_block(1, 70, 7, OVERWORLD).blockstate,
            )

        def test_get_blocks(self):
            selection_box = SelectionBox((0, 0, 0), (10, 10, 10))
            for selection in [SelectionGroup(selection_box), selection_box]:
                chunk, box = next(self.world.get_chunk_boxes(OVERWORLD, selection))
                self.assertIsInstance(chunk, Chunk)
                self.assertIsInstance(box, SelectionBox)

                chunk, slices, _ = next(
                    self.world.get_chunk_slice_box(OVERWORLD, selection)
                )
                self.assertIsInstance(chunk, Chunk)
                self.assertIsInstance(slices, tuple)
                for s in slices:
                    self.assertIsInstance(s, slice)

        def test_clone_operation(self):
            subbx1 = SelectionBox((1, 70, 3), (2, 71, 4))
            src_box = SelectionGroup((subbx1,))

            target = {"x": 1, "y": 70, "z": 5}

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )  # Sanity check
            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

            clone(self.world, OVERWORLD, src_box, target)
            self.world.create_undo_point()

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

            self.world.undo()

            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

            self.world.redo()

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

            self.world.undo()

            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

        def test_fill_operation(self):
            subbox_1 = SelectionBox((1, 70, 3), (5, 71, 5))
            selection = SelectionGroup((subbox_1,))

            # Start sanity check
            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )
            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )
            # End sanity check

            generator_unpacker(
                fill(
                    self.world,
                    OVERWORLD,
                    selection,
                    Block.from_string_blockstate("universal_minecraft:stone"),
                )
            )
            self.world.create_undo_point()

            for x, y, z in selection.blocks:
                self.assertEqual(
                    "universal_minecraft:stone",
                    self.world.get_block(x, y, z, OVERWORLD).blockstate,
                    f"Failed at coordinate ({x},{y},{z})",
                )

            self.world.undo()

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )

            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

            self.world.redo()

            for x, y, z in selection.blocks:
                self.assertEqual(
                    "universal_minecraft:stone",
                    self.world.get_block(x, y, z, OVERWORLD).blockstate,
                    f"Failed at coordinate ({x},{y},{z})",
                )

        def test_replace_single_block(self):
            subbox1 = SelectionBox((1, 70, 3), (5, 71, 6))
            box1 = SelectionGroup((subbox1,))

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )
            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

            replace(
                self.world,
                OVERWORLD,
                box1,
                {
                    "original_blocks": [
                        Block.from_string_blockstate(
                            "universal_minecraft:granite[polished=false]"
                        )
                    ],
                    "replacement_blocks": [
                        Block.from_string_blockstate("universal_minecraft:stone")
                    ],
                },
            )
            self.world.create_undo_point()

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )
            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

            self.world.undo()

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )
            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

            self.world.redo()

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )
            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

        def test_replace_multiblock(self):
            subbox1 = SelectionBox((1, 70, 3), (2, 75, 4))
            box1 = SelectionGroup((subbox1,))

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )
            for y in range(71, 75):
                self.assertEqual(
                    "universal_minecraft:air",
                    self.world.get_block(1, y, 3, OVERWORLD).blockstate,
                    f"Failed at coordinate (1,{y},3)",
                )

            replace(
                self.world,
                OVERWORLD,
                box1,
                {
                    "original_blocks": [
                        Block.from_string_blockstate("universal_minecraft:stone"),
                        Block.from_string_blockstate("universal_minecraft:air"),
                    ],
                    "replacement_blocks": [
                        Block.from_string_blockstate(
                            "universal_minecraft:granite[polished=false]"
                        ),
                        Block.from_string_blockstate("universal_minecraft:stone"),
                    ],
                },
            )
            self.world.create_undo_point()

            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )

            for y in range(71, 75):
                self.assertEqual(
                    "universal_minecraft:stone",
                    self.world.get_block(1, y, 3, OVERWORLD).blockstate,
                    f"Failed at coordinate (1,{y},3)",
                )

            self.world.undo()

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )
            for y in range(71, 75):
                self.assertEqual(
                    "universal_minecraft:air",
                    self.world.get_block(1, y, 3, OVERWORLD).blockstate,
                    f"Failed at coordinate (1,{y},3)",
                )

            self.world.redo()

            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )

            for y in range(71, 75):
                self.assertEqual(
                    "universal_minecraft:stone",
                    self.world.get_block(1, y, 3, OVERWORLD).blockstate,
                    f"Failed at coordinate (1,{y},3)",
                )

        def test_delete_chunk(self):
            subbox1 = SelectionBox((1, 1, 1), (5, 5, 5))
            box1 = SelectionGroup((subbox1,))

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )
            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

            generator_unpacker(delete_chunk(self.world, OVERWORLD, box1))
            self.world.create_undo_point()

            with self.assertRaises(ChunkDoesNotExist):
                _ = self.world.get_block(1, 70, 3, OVERWORLD).blockstate

            self.assertEqual(
                0,
                len([x for x in self.world.get_chunk_slice_box(OVERWORLD, subbox1)]),
            )

            self.world.undo()

            self.assertEqual(
                "universal_minecraft:stone",
                self.world.get_block(1, 70, 3, OVERWORLD).blockstate,
            )
            self.assertEqual(
                "universal_minecraft:granite[polished=false]",
                self.world.get_block(1, 70, 5, OVERWORLD).blockstate,
            )

            self.world.redo()

            with self.assertRaises(ChunkDoesNotExist):
                _ = self.world.get_block(1, 70, 3, OVERWORLD).blockstate

            self.assertEqual(
                0,
                len([x for x in self.world.get_chunk_slice_box(OVERWORLD, subbox1)]),
            )

        @unittest.skipUnless(
            os.path.exists(get_world_path(worlds_src.java_vanilla_1_12_2))
            and os.path.exists(get_world_path(worlds_src.java_vanilla_1_13)),
            reason="Output worlds do not exist",
        )
        def test_save(self):
            version_string = self.world.level_wrapper.game_version_string

            if "1.12.2" in version_string:
                world_name = worlds_src.java_vanilla_1_13
                world_name_temp = f"{worlds_src.java_vanilla_1_12_2} to {worlds_src.java_vanilla_1_13}"
            else:
                world_name = worlds_src.java_vanilla_1_12_2
                world_name_temp = f"{worlds_src.java_vanilla_1_13} to {worlds_src.java_vanilla_1_12_2}"

            output_wrapper = load_format(create_temp_world(world_name, world_name_temp))
            output_wrapper.open()

            self.world.save(output_wrapper)
            self.world.close()
            output_wrapper.close()

            clean_temp_world(world_name_temp)

        @unittest.skip("Entity API currently being rewritten")
        def test_get_entities(
            self,
        ):  # TODO: Make a more complete test once we figure out what get_entities() returns
            box1 = SelectionGroup((SelectionBox((0, 0, 0), (17, 20, 17)),))

            test_entity = {
                "id": "universal_minecraft:cow",
                "CustomName": "TestName",
                "Pos": [1.0, 4.0, 1.0],
            }

            entity_iter = self.world.get_entities_in_box(box1)
            for chunk_coords, entities in entity_iter:
                self.assertEqual(0, len(entities))

            self.world.add_entities([test_entity])

            entity_iter = self.world.get_entities_in_box(box1)
            for chunk_coords, entities in entity_iter:
                if chunk_coords == (0, 0):
                    self.assertEqual(1, len(entities))
                    test_entity["Pos"] = entities[0]["Pos"] = [17.0, 20.0, 17.0]
                else:
                    self.assertEqual(0, len(entities))

            entity_iter = self.world.get_entities_in_box(box1)
            for chunk_coords, entities in entity_iter:
                if chunk_coords == (1, 1):
                    self.assertEqual(1, len(entities))
                else:
                    self.assertEqual(0, len(entities))

            self.world.delete_entities([test_entity])

            entity_iter = self.world.get_entities_in_box(box1)
            for chunk_coords, entities in entity_iter:
                self.assertEqual(0, len(entities))


class AnvilWorldTestCase(WorldTestBaseCases.WorldTestCase):
    def setUp(self):
        self._setUp(worlds_src.java_vanilla_1_12_2)


class Anvil2WorldTestCase(WorldTestBaseCases.WorldTestCase):
    def setUp(self):
        self._setUp(worlds_src.java_vanilla_1_13)


if __name__ == "__main__":
    unittest.main()



import unittest
from amulet.libs.leveldb import LevelDB, LevelDBException
import os
import shutil
import struct

DB_PATH = "./worlds_temp/leveldb"

num_keys = [struct.pack("<Q", i) for i in range(10_000)]
num_db = dict(zip(num_keys, num_keys))
incr_db = {f"key{i}".encode("utf-8"): f"val{i}".encode("utf-8") for i in range(10_000)}
full_db = {**incr_db, **num_db}


class LevelDBTestCase(unittest.TestCase):
    def _clear_db(self):
        if os.path.isdir(DB_PATH):
            shutil.rmtree(DB_PATH)

    def test_create_ldb(self):
        self._clear_db()
        db = LevelDB(DB_PATH, True)
        db.close()

    def test_create_fail(self):
        self._clear_db()
        with self.assertRaises(LevelDBException):
            db = LevelDB(DB_PATH)

    def test_read_write(self):
        self._clear_db()
        db = LevelDB(DB_PATH, True)

        with self.assertRaises(KeyError):
            db.get(b"random_key")

        key1 = b"key"
        value1 = b"value"
        db.put(key1, value1)
        self.assertEqual(db.get(key1), value1)

        key2 = key1 * 1000
        value2 = value1 * 1000
        db.put(key2, value2)
        self.assertEqual(db.get(key2), value2)

        db.close()

    def test_put(self):
        self._clear_db()
        db = LevelDB(DB_PATH, True)

        db.putBatch(incr_db)

        for k, v in num_db.items():
            db.put(k, v)

        self.assertEqual(dict(db.iterate()), full_db)
        # TODO: Uncomment this when the new db wrapper is fixed
        # self.assertEqual(dict(db.items()), full_db)
        self.assertEqual(set(db.keys()), full_db.keys())
        self.assertEqual(set(db), full_db.keys())

        db.close()

    def test_contains(self):
        self._clear_db()
        db = LevelDB(DB_PATH, True)

        self.assertFalse(b"test_key2" in db)

        db.put(b"test_key2", b"test")

        self.assertTrue(b"test_key2" in db)

        db.close()

    def test_delete(self):
        self._clear_db()
        db = LevelDB(DB_PATH, True)

        self.assertFalse(b"test_key3" in db)

        db.put(b"test_key3", b"test")

        self.assertTrue(b"test_key3" in db)

        db.delete(b"test_key3")

        self.assertFalse(b"test_key3" in db)

        db.close()


if __name__ == "__main__":
    unittest.main()



import sys
import os
import time
import shutil
from typing import Optional, Iterable
from contextlib import contextmanager
import re
import json

import tests.data

DATA_DIR = tests.data.__path__[0]


def get_world_path(name: str) -> str:
    return os.path.join(DATA_DIR, "worlds_src", name)


def get_temp_world_path(name: str) -> str:
    return os.path.join(DATA_DIR, "worlds_temp", name)


def get_data_path(name: str) -> str:
    return os.path.join(DATA_DIR, "", name)


class BaseWorldTest:
    WorldPath = ""


def for_each_world(globals_, worlds: Iterable[str]):
    def wrap(cls: BaseWorldTest):
        for world in worlds:
            world_identifier = re.sub("\W|^(?=\d)", "_", world)
            globals_[world_identifier] = type(
                world_identifier, (cls,), {"WorldPath": world}
            )
        return None

    return wrap


class WorldTemp:
    def __init__(self, src_world_name: str, temp_world_name: Optional[str] = None):
        self._src_world_path = get_world_path(src_world_name)
        self._temp_world_path = get_temp_world_path(temp_world_name or src_world_name)
        self._metadata = None

    def __repr__(self):
        return f"WorldTemp({self.src_path}, {self.temp_path})"

    @property
    def src_path(self) -> str:
        return self._src_world_path

    @property
    def temp_path(self) -> str:
        return self._temp_world_path

    @property
    def metadata(self) -> dict:
        if self._metadata is None:
            with open(os.path.join(self.src_path, "world_test_data.json")) as f:
                self._metadata = json.load(f)
        return self._metadata

    def __enter__(self):
        clean_path(self._temp_world_path)
        create_temp_world(self.src_path, self.temp_path)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        clean_path(self.temp_path)


def clean_path(path: str):
    """Clean a given path removing all data at that path."""
    if os.path.isdir(path):
        shutil.rmtree(path, ignore_errors=True)
    elif os.path.isfile(path):
        os.remove(path)


def clean_temp_world(temp_world_name: str) -> str:
    """Remove the temporary world."""
    dst_path = get_temp_world_path(temp_world_name)
    clean_path(dst_path)
    return dst_path


def create_temp_world(
    src_world_name: str, temp_world_name: Optional[str] = None
) -> str:
    """Copy the world to a temporary location and return this path.

    :param src_world_name: The name of a world in ./worlds_src
    :param temp_world_name: Optional temporary name. Leave as None to auto to the same as src
    :return: The full path to a copy of the world
    """
    src_path = get_world_path(src_world_name)
    if temp_world_name is None:
        temp_world_name = src_world_name
    dst_path = clean_temp_world(temp_world_name)

    shutil.copytree(src_path, dst_path)
    return dst_path


@contextmanager
def timeout(test_instance, time_constraint: float, show_completion_time=False):
    start = time.time()
    yield

    end = time.time()
    delta = end - start
    if delta > time_constraint:
        test_instance.fail(
            f"Test execution didn't meet desired run time of {time_constraint}, ran in {delta} instead"
        )
    elif show_completion_time:
        print(
            f"Test ran in {delta} seconds, was required to run in {time_constraint} seconds",
            file=sys.stderr,
        )



import unittest
import numpy

from amulet.utils.matrix import (
    scale_matrix,
    displacement_matrix,
    rotation_matrix_xyz,
    transform_matrix,
    inverse_transform_matrix,
    decompose_transformation_matrix,
)


class MatrixTestCase(unittest.TestCase):
    def test_scale(self):
        numpy.testing.assert_array_equal(
            numpy.matmul(scale_matrix(1, 2, 3), (2, 2, 2, 1))[:3], (2, 4, 6)
        )

    def test_displacement(self):
        numpy.testing.assert_array_equal(
            numpy.matmul(displacement_matrix(1, 2, 3), (2, 2, 2, 1))[:3], (3, 4, 5)
        )

    def test_chain_displacement(self):
        numpy.testing.assert_array_equal(
            numpy.matmul(
                numpy.matmul(
                    displacement_matrix(1, 2, 3),
                    displacement_matrix(1, 2, 3),
                ),
                (2, 2, 2, 1),
            )[:3],
            (4, 6, 8),
        )

    def test_rotation_x(self):
        numpy.testing.assert_array_equal(
            numpy.round(
                numpy.matmul(
                    rotation_matrix_xyz(*numpy.deg2rad([90, 0, 0])), (1, 1, 1, 1)
                )[:3],
                1,
            ),
            (1, -1, 1),
        )

    def test_rotation_y(self):
        numpy.testing.assert_array_equal(
            numpy.round(
                numpy.matmul(
                    rotation_matrix_xyz(*numpy.deg2rad([0, 90, 0])), (1, 1, 1, 1)
                )[:3],
                1,
            ),
            (1, 1, -1),
        )

    def test_rotation_z(self):
        numpy.testing.assert_array_equal(
            numpy.round(
                numpy.matmul(
                    rotation_matrix_xyz(*numpy.deg2rad([0, 0, 90])), (1, 1, 1, 1)
                )[:3],
                1,
            ),
            (-1, 1, 1),
        )

    def test_chain_rotation(self):
        numpy.testing.assert_array_equal(
            numpy.round(
                numpy.matmul(
                    numpy.matmul(
                        rotation_matrix_xyz(*numpy.deg2rad([0, 90, 0])),
                        rotation_matrix_xyz(*numpy.deg2rad([0, 90, 0])),
                    ),
                    (1, 1, 1, 1),
                )[:3],
                1,
            ),
            (-1, 1, -1),
        )

    def test_chain_rotation_xy(self):
        numpy.testing.assert_array_equal(
            numpy.round(
                numpy.matmul(
                    numpy.matmul(
                        rotation_matrix_xyz(
                            *numpy.deg2rad([0, 90, 0])
                        ),  # y rotation applied second
                        rotation_matrix_xyz(
                            *numpy.deg2rad([90, 0, 0])
                        ),  # x rotation applied first
                    ),
                    (1, 1, 1, 1),
                )[:3],
                1,
            ),
            (1, -1, -1),
        )

    def test_rotation_xyz(self):
        numpy.testing.assert_array_equal(
            numpy.round(
                numpy.matmul(
                    rotation_matrix_xyz(*numpy.deg2rad([90, 90, 0])), (1, 1, 1, 1)
                )[:3],
                1,
            ),
            (1, -1, -1),
        )

    def test_transform(self):
        numpy.testing.assert_array_equal(
            numpy.round(
                numpy.matmul(
                    transform_matrix((1, 1, 1), (0, 0, 0), (10, 20, 30)), (1, 1, 1, 1)
                )[:3],
                1,
            ),
            (11, 21, 31),
        )

    def test_chain_transform(self):
        numpy.testing.assert_array_equal(
            numpy.round(
                numpy.matmul(
                    numpy.matmul(
                        transform_matrix(
                            (1, 1, 1), (0, numpy.deg2rad(90), 0), (0, 0, 0)
                        ),
                        transform_matrix((1, 1, 1), (0, 0, 0), (10, 20, 30)),
                    ),
                    (1, 1, 1, 1),
                )[:3],
                1,
            ),
            (31, 21, -11),
        )

    def test_chain_transform2(self):
        numpy.testing.assert_array_equal(
            numpy.round(
                numpy.matmul(
                    numpy.matmul(
                        transform_matrix((1, 1, 1), (0, 0, 0), (10, 20, 30)),
                        transform_matrix(
                            (1, 1, 1), (0, numpy.deg2rad(90), 0), (0, 0, 0)
                        ),
                    ),
                    (1, 1, 1, 1),
                )[:3],
                1,
            ),
            (11, 21, 29),
        )

    def test_inverse_transform(self):
        inputs = numpy.random.random(9).reshape((3, 3)).tolist()
        transform = transform_matrix(*inputs)
        inverse_transform = inverse_transform_matrix(*inputs)
        numpy.testing.assert_array_almost_equal(
            numpy.matmul(transform, inverse_transform), numpy.eye(4)
        )

    def test_decompose(self):
        rotation_ = (1, 1.5, 3)
        m = rotation_matrix_xyz(*rotation_)
        scale, rotation, displacement = decompose_transformation_matrix(m)
        numpy.testing.assert_array_almost_equal(
            displacement, (0, 0, 0), err_msg="decomposed displacement is incorrect"
        )
        numpy.testing.assert_array_almost_equal(
            scale, (1, 1, 1), err_msg="decomposed scale is incorrect"
        )
        numpy.testing.assert_array_almost_equal(
            rotation, rotation_, err_msg="decomposed rotation is incorrect"
        )

        m = transform_matrix((1, 2, 3), rotation_, (1, 2, 3))
        scale, rotation, displacement = decompose_transformation_matrix(m)
        numpy.testing.assert_array_almost_equal(
            displacement, (1, 2, 3), err_msg="decomposed displacement is incorrect"
        )
        numpy.testing.assert_array_almost_equal(
            scale, (1, 2, 3), err_msg="decomposed scale is incorrect"
        )
        numpy.testing.assert_array_almost_equal(
            rotation, rotation_, err_msg="decomposed rotation is incorrect"
        )


if __name__ == "__main__":
    unittest.main()



import unittest

from amulet.api.errors import PlayerDoesNotExist
from amulet.api.player import Player
from amulet import load_level
from tests.data.util import create_temp_world, clean_temp_world
from tests.data import worlds_src


class WorldTestBaseCases:
    # Wrapped in another class, so it isn't executed on it's own.

    class WorldTestCase(unittest.TestCase):
        def _setUp(self, world_name):
            self._world_name = world_name
            self.world = load_level(create_temp_world(world_name))

        def tearDown(self):
            self.world.close()
            clean_temp_world(self._world_name)

        def test_get_players(self):
            player_ids = list(self.world.all_player_ids())
            self.assertGreaterEqual(len(player_ids), 1)
            for player_id in player_ids:
                self.assertTrue(self.world.has_player(player_id))
                player = self.world.get_player(player_id)
                self.assertIsInstance(player, Player)

            with self.assertRaises(PlayerDoesNotExist):
                self.world.get_player("test")


def _create_test(path):
    cls_name = f"Test{path}"

    class TestCase(WorldTestBaseCases.WorldTestCase):
        def setUp(self):
            self._setUp(path)

    globals()[cls_name] = TestCase


for _path in worlds_src.levels:
    _create_test(_path)


if __name__ == "__main__":
    unittest.main()
